
/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache POI" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache POI", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.poi.hssf.record.formula;

import org.apache.poi.util.LittleEndian;
import org.apache.poi.hssf.util.RangeAddress;

/**
 * Title:        Area 3D Ptg <P>
 * Description:  Defined a area in Extern Sheet. <P>
 * REFERENCE:  <P>
 * @author Libin Roman (Vista Portal LDT. Developer)
 * @version 1.0-pre
 */

public class Area3DPtg extends Ptg
{
    public final static short sid  = 0x3b;
    private final static int  SIZE = 11; // 10 + 1 for Ptg
    private short             field_1_index_extern_sheet;  
    private short             field_2_first_row;
    private short             field_3_last_row;
    private short             field_4_first_column;
    private short             field_5_last_column;

    /** Creates new AreaPtg */
    public Area3DPtg()
    {
    }

    /**
     * Constructs a Area 3D Ptg and sets its fields appropriately.
     *
     * @param size  the size of the data area of the record
     * @param data  data of the record (should not contain sid/len)
     */
    public Area3DPtg(byte[] data, int offset)
    {
        offset++;
        field_1_index_extern_sheet = LittleEndian.getShort(data, 0 + offset);
        field_2_first_row          = LittleEndian.getShort(data, 2 + offset);
        field_3_last_row           = LittleEndian.getShort(data, 4 + offset);
        field_4_first_column       = LittleEndian.getShort(data, 6 + offset);
        field_5_last_column        = LittleEndian.getShort(data, 8 + offset);
    }

    /**
     * @return A string represent of the ptg
     */    
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("AreaPtg\n");
        buffer.append("Index to Extern Sheet = " + getExternSheetIndex()).append("\n");
        buffer.append("firstRow = " + getFirstRow()).append("\n");
        buffer.append("lastRow  = " + getLastRow()).append("\n");
        buffer.append("firstCol = " + getFirstColumn()).append("\n");
        buffer.append("lastCol  = " + getLastColumn()).append("\n");
        buffer.append("firstColRowRel= "
                      + isFirstColRowRelative()).append("\n");
        buffer.append("lastColRowRel = "
                      + isLastColRowRelative()).append("\n");
        buffer.append("firstColRel   = " + isFirstColRelative()).append("\n");
        buffer.append("lastColRel    = " + isLastColRelative()).append("\n");
        return buffer.toString();
    }

    /**
     * called by the class that is responsible for writing this sucker.
     * Subclasses should implement this so that their data is passed back in a
     * byte array.
     *
     * @param array the data
     * @param offset to begin writing at */
    public void writeBytes(byte [] array, int offset)
    {
        array[ 0 + offset ] = sid;
        LittleEndian.putShort(array, 1 + offset , getExternSheetIndex());
        LittleEndian.putShort(array, 3 + offset , getFirstRow());
        LittleEndian.putShort(array, 5 + offset , getLastRow());               
        LittleEndian.putShort(array, 7 + offset , getFirstColumnRaw());               
        LittleEndian.putShort(array, 9 + offset , getLastColumnRaw());               
    }

    /** gets the size of the ptg
     *
     * @return  ptg size */    
    public int getSize()
    {
        return SIZE;
    }
    
    /** get the extern sheet Index,
     * the index to the extern sheet that
     * represented the reference to sheet
     * @return extern sheet index */    
    public short getExternSheetIndex(){
        return field_1_index_extern_sheet;
    }
    
    /** sets the extern sheet index
     *
     * @param index  extern sheet index*/    
    public void setExternSheetIndex(short index){
        field_1_index_extern_sheet = index;
    }

    /** gets the first row of the area
     *
     * @return  first row of area */    
    public short getFirstRow()
    {
        return field_2_first_row;
    }

    /** sets the first row of the area
     *
     * @param row  first row of area*/    
    public void setFirstRow(short row)
    {
        field_2_first_row = row;
    }

    /** gets the last row of the area
     *
     * @return  last row of the area */    
    public short getLastRow()
    {
        return field_3_last_row;
    }

    /** sets the last row of the area
     *
     * @param row  last row */    
    public void setLastRow(short row)
    {
        field_3_last_row = row;
    }

    /** returns the first column of the area (without flags)
     *
     * @return  first column of area */    
    public short getFirstColumn()
    {
        return ( short ) (field_4_first_column & 0xFF);
    }

    /** returns the first column of the area (with flags)
     * @return  first column of area */    
    public short getFirstColumnRaw()
    {
        return field_4_first_column;
    }

    /** returns if the first column is relative
     *
     * @return  if first row relative */    
    public boolean isFirstColRowRelative()
    {
        return (((getFirstColumnRaw()) & 0x8000) == 0x8000);
    }

    /** returns if the first column is relative
     *
     * @return  if first row relative */    
    public boolean isFirstColRelative()
    {
        return (((getFirstColumnRaw()) & 0x4000) == 0x4000);
    }

    /** sets the first column of the area (without flags)
     *
     * @param column first column of area
     */    
    public void setFirstColumn(short column)
    {
        field_4_first_column &= 0xFF00;
        field_4_first_column |= column & 0xFF; 
    }

    /** returns the first column of the area (with flags)
     *
     * @param column first column of area
     */    
    public void setFirstColumnRaw(short column)
    {
        field_4_first_column = column;   
    }

    /** returns the last column of the area (without flags)
     *
     * @return  last column of area */    
    public short getLastColumn()
    {
        return ( short ) (field_5_last_column & 0xFF);   
    }

    /** returns the last column of the area (with flags)
     *
     * @return  last column of area */    
    public short getLastColumnRaw()
    {
        return field_5_last_column;
    }

    /** returns if the last column is relative
     *
     * @return  if last row relative */    
    public boolean isLastColRowRelative()
    {
        return (((getLastColumnRaw()) & 0x8000) == 1);
    }

    /** returns if the last column is relative
     *
     * @return  if last row relative */    
    public boolean isLastColRelative()
    {
        return (((getFirstColumnRaw()) & 0x4000) == 1);
    }

    /** sets the last column of the area (without flags)
     *
     * @param column last column of area
     */    
    public void setLastColumn(short column)
    {
        field_5_last_column &= 0xFF00;
        field_5_last_column |= column & 0xFF; 
    }

    /** sets the last column of the area (with flags)
     *
     * @param column last column of area
     */    
    public void setLastColumnRaw(short column)
    {
        field_5_last_column = column;
    }
    
    /** returns the reference (area)
     *
     * @return  reference */    
    public String getArea(){
        RangeAddress ra = new RangeAddress( getFirstColumn() + 1,getFirstRow() + 1, getLastColumn() + 1, getLastRow() + 1);   
        String result = ra.getAddress();
        
        return result;
    }
    
    /** sets the reference (area)
     *
     * @param ref reference (area)
     */    
    public void setArea(String ref){
        RangeAddress ra = new RangeAddress(ref);
        
        String from = ra.getFromCell();
        String to   = ra.getToCell();
        
        setFirstColumn((short) (ra.getXPosition(from) -1));
        setFirstRow((short) (ra.getYPosition(from) -1));
        setLastColumn((short) (ra.getXPosition(to) -1));
        setLastRow((short) (ra.getYPosition(to) -1));
                        
    }

    /** returns the string reprecent of the formula 
     *
     * @return  formula reprecent */    
    public String toFormulaString()
    {
        String result = getArea();

        return result;
    }

    
}
